from django.db import models
from django_dynamic_from_clause.query import DynamicFromClauseQuerySet


class DynamicFromClauseBaseModel(models.Model):
    """
    Require to set source queryset or expression before evaluation.
    In fact, can be evaluated from anything which is QuerySet/Expression, and meet declared interface.
    """
    EXPRESSION_CLASS: None
    objects = DynamicFromClauseQuerySet.as_manager()

    class Meta:
        app_label = ''
        abstract = True
        managed = False
        db_table: str



from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.db.models import QuerySet, Func

from django_dynamic_from_clause.models import DynamicFromClauseBaseModel


# Example models
from django_dynamic_from_clause.query import DynamicFromClauseQuerySet


class Owner(models.Model):
    name = models.CharField(max_length=512)


class InventoryRecord(models.Model):
    count = models.IntegerField()
    owner = models.ForeignKey(Owner, related_name='inventory_records', on_delete=models.CASCADE)


# regular table which using Dynamic Manager
class Human(models.Model):
    objects = QuerySet.as_manager()
    dynamic_from_clause_objects = DynamicFromClauseQuerySet.as_manager()
    weight = models.IntegerField()
    height = models.IntegerField()


# Example perspectives for QuerySets
class AggregatedInventoryPerspective(DynamicFromClauseBaseModel):
    count_sum = models.IntegerField()
    owner = models.ForeignKey(Owner, related_name='+', on_delete=models.DO_NOTHING, primary_key=True)


# Already declared Table function
class InvRecordTableFunctionModel(DynamicFromClauseBaseModel):
    id = models.IntegerField(InventoryRecord, primary_key=True)


# Example functions which returns single value
class PgBackendPidFunc(Func):
    function = 'pg_backend_pid'
    template = "%(function)s(%(expressions)s)"
    arity = 0


class PgBackendPid(DynamicFromClauseBaseModel):
    EXPRESSION_CLASS = PgBackendPidFunc
    _pgbackendpid = models.IntegerField(primary_key=True)


class PgBlockingPidsModel(DynamicFromClauseBaseModel):
    _pgblockingpidsmodel = ArrayField(base_field=models.IntegerField(), primary_key=True)




import django
from django.db import connections
from django.db.models.sql import Query

from django_dynamic_from_clause.compiler import DynamicFormClauseSQLCompiler


class DynamicFormClauseQuery(Query):
    compiler = DynamicFormClauseSQLCompiler

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.source = None

    def get_compiler(self, using=None, connection=None, elide_empty=True):
        if using is None and connection is None:
            raise ValueError("Need either using or connection")
        if using:
            connection = connections[using]
        if django.VERSION >= (4,):
            return self.compiler(self, connection, using, elide_empty)
        return self.compiler(self, connection, using)
